<?php

namespace App\Console\Commands\Video\WechatCheck;

use App\Service\Miniprogram\Wechat\AccessTokenService;
use App\Service\Util\Support\Http\WechatURL;
use App\Service\Util\Support\Http\HttpRequestService;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;

class SyncDramaInfo extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'WechatCheck:SyncDramaInfo {--drama_ids= : 剧目ids}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = '微信剧目提审-获取剧目信息';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        $drama_ids = $this->option('drama_ids');
        $dramaIds = null;
        if($drama_ids) {
            $dramaIds = explode(',', $drama_ids);
        }
        DB::table('video_wechat_check')
            ->when($dramaIds, function ($query, $dramaIds) {
                return $query->whereIn('drama_id', $dramaIds);
            })
            ->where('drama_id', '<>', 0)
            ->where([
                'status' => 1,
                'is_enabled' => 1
            ])
            ->select('drama_id', 'id', 'status', 'appid', 'video_id')
            ->orderBy('id')
            ->chunk(100, function ($items) {
                foreach ($items as $item) {
                    $accessToken = $this->getAccessToken($item->appid ?: config('wechat.duanju.masterAppid'));
                    $result = HttpRequestService::simplePost(WechatURL::vod_getdrama. $accessToken, [
                        'drama_id' => $item->drama_id
                    ]);
                    if(false === $result || (0 != ($result['errcode'] ?? 0))) {
                        myLog('SyncDramaInfo')->error('获取剧目信息失败', [
                            'id' => $item->id, 'drama_id' => $item->drama_id,
                            'result' => $result,
                        ]);
                        continue;
                    }
                    $status = $result['drama_info']['audit_detail']['status'];
                    $now = date('Y-m-d H:i:s');
                    if($status != $item->status)
                    {
                        DB::table('video_wechat_check')
                            ->where('id', $item->id)
                            ->update([
                                'check_at' => date('Y-m-d H:i:s', $result['drama_info']['audit_detail']['audit_time']),
                                'status' => $status,
                                'updated_at' => $now,
                            ]);
                    }
                    if(3 == $status) {
                        DB::table('videos')
                            ->where('id', $item->video_id)
                            ->update([
                                'wechat_pass' => 1,
                                'updated_at' => $now,
                            ]);
                    }
                }
            });
    }

    private function getAccessToken($appid) {
        return Redis::get(AccessTokenService::getAccessTokenRedisKey($appid));
    }
}
